@@ -1,6 +1,9 @@
 use crate::{
-    accounts_db::{AccountsDb, BankHashInfo, ErrorCounters, LoadedAccount, ScanStorageResult},
-    accounts_index::{AccountSecondaryIndexes, Ancestors, IndexKey},
+    accounts_db::{
+        AccountsDb, BankHashInfo, ErrorCounters, LoadHint, LoadedAccount, ScanStorageResult,
+    },
+    accounts_index::{AccountSecondaryIndexes, IndexKey},
+    ancestors::Ancestors,
     bank::{
         NonceRollbackFull, NonceRollbackInfo, RentDebits, TransactionCheckResult,
         TransactionExecutionResult,
@@ -16,17 +19,16 @@ use dashmap::{
 use log::*;
 use rand::{thread_rng, Rng};
 use solana_sdk::{
-    account::{Account, AccountSharedData},
+    account::{Account, AccountSharedData, ReadableAccount, WritableAccount},
     account_utils::StateMut,
     bpf_loader_upgradeable::{self, UpgradeableLoaderState},
     clock::{Slot, INITIAL_RENT_EPOCH},
     feature_set::{self, FeatureSet},
-    fee_calculator::FeeCalculator,
+    fee_calculator::{FeeCalculator, FeeConfig},
     genesis_config::ClusterType,
     hash::Hash,
-    message::Message,
+    message::{Message, MessageProgramIdsCache},
     native_loader, nonce,
-    nonce::NONCED_TX_MARKER_IX_INDEX,
     pubkey::Pubkey,
     transaction::Result,
     transaction::{Transaction, TransactionError},
@@ -205,10 +207,10 @@ impl Accounts {
             let mut account_deps = Vec::with_capacity(message.account_keys.len());
             let demote_sysvar_write_locks =
                 feature_set.is_active(&feature_set::demote_sysvar_write_locks::id());
+            let mut key_check = MessageProgramIdsCache::new(&message);
             let mut rent_debits = RentDebits::default();
-
             for (i, key) in message.account_keys.iter().enumerate() {
-                let account = if message.is_non_loader_key(key, i) {
+                let account = if key_check.is_non_loader_key(key, i) {
                     if payer_index.is_none() {
                         payer_index = Some(i);
                     }
@@ -223,7 +225,7 @@ impl Accounts {
                     } else {
                         let (account, rent) = self
                             .accounts_db
-                            .load(ancestors, key)
+                            .load_with_fixed_root(ancestors, key)
                             .map(|(mut account, _)| {
                                 if message.is_writable(i, demote_sysvar_write_locks) {
                                     let rent_due = rent_collector
@@ -235,7 +237,8 @@ impl Accounts {
                             })
                             .unwrap_or_default();
 
-                        if account.executable && bpf_loader_upgradeable::check_id(&account.owner) {
+                        if account.executable() && bpf_loader_upgradeable::check_id(account.owner())
+                        {
                             // The upgradeable loader requires the derived ProgramData account
                             if let Ok(UpgradeableLoaderState::Program {
                                 programdata_address,
@@ -243,7 +246,7 @@ impl Accounts {
                             {
                                 if let Some(account) = self
                                     .accounts_db
-                                    .load(ancestors, &programdata_address)
+                                    .load_with_fixed_root(ancestors, &programdata_address)
                                     .map(|(account, _)| account)
                                 {
                                     account_deps.push((programdata_address, account));
@@ -258,7 +261,7 @@ impl Accounts {
                         }
 
                         tx_rent += rent;
-                        rent_debits.push(key, rent, account.lamports);
+                        rent_debits.push(key, rent, account.lamports());
 
                         account
                     }
@@ -274,7 +277,7 @@ impl Accounts {
                 if payer_index != 0 {
                     warn!("Payer index should be 0! {:?}", tx);
                 }
-                if accounts[payer_index].lamports == 0 {
+                if accounts[payer_index].lamports() == 0 {
                     error_counters.account_not_found += 1;
                     Err(TransactionError::AccountNotFound)
                 } else {
@@ -291,11 +294,13 @@ impl Accounts {
                         }
                     };
 
-                    if accounts[payer_index].lamports < fee + min_balance {
+                    if accounts[payer_index].lamports() < fee + min_balance {
                         error_counters.insufficient_funds += 1;
                         Err(TransactionError::InsufficientFundsForFee)
                     } else {
-                        accounts[payer_index].lamports -= fee;
+                        accounts[payer_index]
+                            .checked_sub_lamports(fee)
+                            .map_err(|_| TransactionError::InsufficientFundsForFee)?;
 
                         let message = tx.message();
                         let loaders = message
@@ -353,7 +358,7 @@ impl Accounts {
 
             let program = match self
                 .accounts_db
-                .load(ancestors, &program_id)
+                .load_with_fixed_root(ancestors, &program_id)
                 .map(|(account, _)| account)
             {
                 Some(program) => program,
@@ -362,13 +367,13 @@ impl Accounts {
                     return Err(TransactionError::ProgramAccountNotFound);
                 }
             };
-            if !program.executable {
+            if !program.executable() {
                 error_counters.invalid_program_for_execution += 1;
                 return Err(TransactionError::InvalidProgramForExecution);
             }
 
             // Add loader to chain
-            let program_owner = program.owner;
+            let program_owner = *program.owner();
 
             if bpf_loader_upgradeable::check_id(&program_owner) {
                 // The upgradeable loader requires the derived ProgramData account
@@ -378,7 +383,7 @@ impl Accounts {
                 {
                     if let Some(program) = self
                         .accounts_db
-                        .load(ancestors, &programdata_address)
+                        .load_with_fixed_root(ancestors, &programdata_address)
                         .map(|(account, _)| account)
                     {
                         accounts.insert(0, (programdata_address, program));
@@ -408,6 +413,10 @@ impl Accounts {
         rent_collector: &RentCollector,
         feature_set: &FeatureSet,
     ) -> Vec<TransactionLoadResult> {
+        let fee_config = FeeConfig {
+            secp256k1_program_enabled: feature_set
+                .is_active(&feature_set::secp256k1_program_enabled::id()),
+        };
         txs.zip(lock_results)
             .map(|etx| match etx {
                 (tx, (Ok(()), nonce_rollback)) => {
@@ -420,7 +429,7 @@ impl Accounts {
                                 .cloned()
                         });
                     let fee = if let Some(fee_calculator) = fee_calculator {
-                        fee_calculator.calculate_fee(tx.message())
+                        fee_calculator.calculate_fee_with_config(tx.message(), &fee_config)
                     } else {
                         return (Err(TransactionError::BlockhashNotFound), None);
                     };
@@ -458,21 +467,44 @@ impl Accounts {
             .collect()
     }
 
-    /// Slow because lock is held for 1 operation instead of many
-    pub fn load_slow(
-        &self,
-        ancestors: &Ancestors,
-        pubkey: &Pubkey,
+    fn filter_zero_lamport_account(
+        account: AccountSharedData,
+        slot: Slot,
     ) -> Option<(AccountSharedData, Slot)> {
-        let (account, slot) = self.accounts_db.load_slow(ancestors, pubkey)?;
-
-        if account.lamports > 0 {
+        if account.lamports() > 0 {
             Some((account, slot))
         } else {
             None
         }
     }
 
+    /// Slow because lock is held for 1 operation instead of many
+    fn load_slow(
+        &self,
+        ancestors: &Ancestors,
+        pubkey: &Pubkey,
+        load_hint: LoadHint,
+    ) -> Option<(AccountSharedData, Slot)> {
+        let (account, slot) = self.accounts_db.load(ancestors, pubkey, load_hint)?;
+        Self::filter_zero_lamport_account(account, slot)
+    }
+
+    pub fn load_with_fixed_root(
+        &self,
+        ancestors: &Ancestors,
+        pubkey: &Pubkey,
+    ) -> Option<(AccountSharedData, Slot)> {
+        self.load_slow(ancestors, pubkey, LoadHint::FixedMaxRoot)
+    }
+
+    pub fn load_without_fixed_root(
+        &self,
+        ancestors: &Ancestors,
+        pubkey: &Pubkey,
+    ) -> Option<(AccountSharedData, Slot)> {
+        self.load_slow(ancestors, pubkey, LoadHint::Unspecified)
+    }
+
     /// scans underlying accounts_db for this delta (slot) with a map function
     ///   from LoadedAccount to B
     /// returns only the latest/current version of B for this slot
@@ -535,7 +567,7 @@ impl Accounts {
             };
 
             if hit {
-                Some((*stored_account.pubkey(), stored_account.account()))
+                Some((*stored_account.pubkey(), stored_account.take_account()))
             } else {
                 None
             }
@@ -556,7 +588,7 @@ impl Accounts {
             ancestors,
             |collector: &mut BinaryHeap<Reverse<(u64, Pubkey)>>, option| {
                 if let Some((pubkey, account, _slot)) = option {
-                    if account.lamports == 0 {
+                    if account.lamports() == 0 {
                         return;
                     }
                     let contains_address = filter_by_address.contains(pubkey);
@@ -571,12 +603,12 @@ impl Accounts {
                         let Reverse(entry) = collector
                             .peek()
                             .expect("BinaryHeap::peek should succeed when len > 0");
-                        if *entry >= (account.lamports, *pubkey) {
+                        if *entry >= (account.lamports(), *pubkey) {
                             return;
                         }
                         collector.pop();
                     }
-                    collector.push(Reverse((account.lamports, *pubkey)));
+                    collector.push(Reverse((account.lamports(), *pubkey)));
                 }
             },
         );
@@ -594,15 +626,9 @@ impl Accounts {
             |total_capitalization: &mut u64, (_pubkey, loaded_account, _slot)| {
                 let lamports = loaded_account.lamports();
                 if Self::is_loadable(lamports) {
-                    let account_cap = AccountsDb::account_balance_for_capitalization(
-                        lamports,
-                        &loaded_account.owner(),
-                        loaded_account.executable(),
-                    );
-
                     *total_capitalization = AccountsDb::checked_iterative_sum_for_capitalization(
                         *total_capitalization,
-                        account_cap,
+                        lamports,
                     );
                 }
             },
@@ -639,7 +665,7 @@ impl Accounts {
         filter: F,
     ) {
         if let Some(mapped_account_tuple) = some_account_tuple
-            .filter(|(_, account, _)| Self::is_loadable(account.lamports) && filter(account))
+            .filter(|(_, account, _)| Self::is_loadable(account.lamports()) && filter(account))
             .map(|(pubkey, account, _slot)| (*pubkey, account))
         {
             collector.push(mapped_account_tuple)
@@ -655,7 +681,7 @@ impl Accounts {
             ancestors,
             |collector: &mut Vec<(Pubkey, AccountSharedData)>, some_account_tuple| {
                 Self::load_while_filtering(collector, some_account_tuple, |account| {
-                    account.owner == *program_id
+                    account.owner() == program_id
                 })
             },
         )
@@ -671,7 +697,7 @@ impl Accounts {
             ancestors,
             |collector: &mut Vec<(Pubkey, AccountSharedData)>, some_account_tuple| {
                 Self::load_while_filtering(collector, some_account_tuple, |account| {
-                    account.owner == *program_id && filter(account)
+                    account.owner() == program_id && filter(account)
                 })
             },
         )
@@ -704,8 +730,8 @@ impl Accounts {
         self.accounts_db.scan_accounts(
             ancestors,
             |collector: &mut Vec<(Pubkey, AccountSharedData, Slot)>, some_account_tuple| {
-                if let Some((pubkey, account, slot)) =
-                    some_account_tuple.filter(|(_, account, _)| Self::is_loadable(account.lamports))
+                if let Some((pubkey, account, slot)) = some_account_tuple
+                    .filter(|(_, account, _)| Self::is_loadable(account.lamports()))
                 {
                     collector.push((*pubkey, account, slot))
                 }
@@ -814,6 +840,7 @@ impl Accounts {
     /// This function will prevent multiple threads from modifying the same account state at the
     /// same time
     #[must_use]
+    #[allow(clippy::needless_collect)]
     pub fn lock_accounts<'a>(
         &self,
         txs: impl Iterator<Item = &'a Transaction>,
@@ -876,7 +903,6 @@ impl Accounts {
         last_blockhash_with_fee_calculator: &(Hash, FeeCalculator),
         fix_recent_blockhashes_sysvar_delay: bool,
         demote_sysvar_write_locks: bool,
-        merge_nonce_error_into_system_error: bool,
     ) {
         let accounts_to_store = self.collect_accounts_to_store(
             txs,
@@ -886,15 +912,15 @@ impl Accounts {
             last_blockhash_with_fee_calculator,
             fix_recent_blockhashes_sysvar_delay,
             demote_sysvar_write_locks,
-            merge_nonce_error_into_system_error,
         );
         self.accounts_db.store_cached(slot, &accounts_to_store);
     }
 
     /// Purge a slot if it is not a root
     /// Root slots cannot be purged
-    pub fn purge_slot(&self, slot: Slot) {
-        self.accounts_db.purge_slot(slot);
+    /// `is_from_abs` is true if the caller is the AccountsBackgroundService
+    pub fn purge_slot(&self, slot: Slot, is_from_abs: bool) {
+        self.accounts_db.purge_slot(slot, is_from_abs);
     }
 
     /// Add a slot to root.  Root slots cannot be purged
@@ -911,7 +937,6 @@ impl Accounts {
         last_blockhash_with_fee_calculator: &(Hash, FeeCalculator),
         fix_recent_blockhashes_sysvar_delay: bool,
         demote_sysvar_write_locks: bool,
-        merge_nonce_error_into_system_error: bool,
     ) -> Vec<(&'a Pubkey, &'a AccountSharedData)> {
         let mut accounts = Vec::with_capacity(loaded.len());
         for (i, ((raccs, _nonce_rollback), tx)) in loaded.iter_mut().zip(txs).enumerate() {
@@ -924,19 +949,13 @@ impl Accounts {
                     let pubkey = nonce_rollback.nonce_address();
                     let acc = nonce_rollback.nonce_account();
                     let maybe_fee_account = nonce_rollback.fee_account();
-                    Some((pubkey, acc, maybe_fee_account, true))
+                    Some((pubkey, acc, maybe_fee_account))
                 }
-                (Err(TransactionError::InstructionError(index, _)), Some(nonce_rollback)) => {
-                    let nonce_marker_ix_failed = if merge_nonce_error_into_system_error {
-                        // Don't advance stored blockhash when the nonce marker ix fails
-                        *index == NONCED_TX_MARKER_IX_INDEX
-                    } else {
-                        false
-                    };
+                (Err(TransactionError::InstructionError(_, _)), Some(nonce_rollback)) => {
                     let pubkey = nonce_rollback.nonce_address();
                     let acc = nonce_rollback.nonce_account();
                     let maybe_fee_account = nonce_rollback.fee_account();
-                    Some((pubkey, acc, maybe_fee_account, !nonce_marker_ix_failed))
+                    Some((pubkey, acc, maybe_fee_account))
                 }
                 (Ok(_), _nonce_rollback) => None,
                 (Err(_), _nonce_rollback) => continue,
@@ -971,22 +990,22 @@ impl Accounts {
                     if res.is_err() {
                         match (is_nonce_account, is_fee_payer, maybe_nonce_rollback) {
                             // nonce is fee-payer, state updated in `prepare_if_nonce_account()`
-                            (true, true, Some((_, _, None, _))) => (),
+                            (true, true, Some((_, _, None))) => (),
                             // nonce not fee-payer, state updated in `prepare_if_nonce_account()`
-                            (true, false, Some((_, _, Some(_), _))) => (),
+                            (true, false, Some((_, _, Some(_)))) => (),
                             // not nonce, but fee-payer. rollback to cached state
-                            (false, true, Some((_, _, Some(fee_payer_account), _))) => {
+                            (false, true, Some((_, _, Some(fee_payer_account)))) => {
                                 *account = fee_payer_account.clone();
                             }
                             _ => panic!("unexpected nonce_rollback condition"),
                         }
                     }
-                    if account.rent_epoch == INITIAL_RENT_EPOCH {
+                    if account.rent_epoch() == INITIAL_RENT_EPOCH {
                         let rent = rent_collector.collect_from_created_account(&key, account);
                         loaded_transaction.rent += rent;
                         loaded_transaction
                             .rent_debits
-                            .push(key, rent, account.lamports);
+                            .push(key, rent, account.lamports());
                     }
                     accounts.push((key, &*account));
                 }
@@ -1000,18 +1019,11 @@ pub fn prepare_if_nonce_account(
     account: &mut AccountSharedData,
     account_pubkey: &Pubkey,
     tx_result: &Result<()>,
-    maybe_nonce_rollback: Option<(
-        &Pubkey,
-        &AccountSharedData,
-        Option<&AccountSharedData>,
-        bool,
-    )>,
+    maybe_nonce_rollback: Option<(&Pubkey, &AccountSharedData, Option<&AccountSharedData>)>,
     last_blockhash_with_fee_calculator: &(Hash, FeeCalculator),
     fix_recent_blockhashes_sysvar_delay: bool,
 ) -> bool {
-    if let Some((nonce_key, nonce_acc, _maybe_fee_account, advance_blockhash)) =
-        maybe_nonce_rollback
-    {
+    if let Some((nonce_key, nonce_acc, _maybe_fee_account)) = maybe_nonce_rollback {
         if account_pubkey == nonce_key {
             let overwrite = if tx_result.is_err() {
                 // Nonce TX failed with an InstructionError. Roll back
@@ -1023,10 +1035,7 @@ pub fn prepare_if_nonce_account(
                 // recent_blockhashes_sysvar_delay fix is activated
                 !fix_recent_blockhashes_sysvar_delay
             };
-            if overwrite && advance_blockhash {
-                // Advance the stored blockhash to prevent fee theft by replaying
-                // transactions that have failed with an `InstructionError`
-
+            if overwrite {
                 // Since hash_age_kind is DurableNonce, unwrap is safe here
                 let state = StateMut::<nonce::state::Versions>::state(nonce_acc)
                     .unwrap()
@@ -1057,7 +1066,7 @@ pub fn create_test_accounts(
     for t in 0..num {
         let pubkey = solana_sdk::pubkey::new_rand();
         let account =
-            AccountSharedData::new((t + 1) as u64, 0, &AccountSharedData::default().owner);
+            AccountSharedData::new((t + 1) as u64, 0, AccountSharedData::default().owner());
         accounts.store_slow_uncached(slot, &pubkey, &account);
         pubkeys.push(pubkey);
     }
@@ -1068,7 +1077,7 @@ pub fn create_test_accounts(
 pub fn update_accounts_bench(accounts: &Accounts, pubkeys: &[Pubkey], slot: u64) {
     for pubkey in pubkeys {
         let amount = thread_rng().gen_range(0, 10);
-        let account = AccountSharedData::new(amount, 0, &AccountSharedData::default().owner);
+        let account = AccountSharedData::new(amount, 0, AccountSharedData::default().owner());
         accounts.store_slow_uncached(slot, &pubkey, &account);
     }
 }
@@ -1339,10 +1348,10 @@ mod tests {
         assert_eq!(loaded_accounts.len(), 1);
         let (load_res, _nonce_rollback) = &loaded_accounts[0];
         let loaded_transaction = load_res.as_ref().unwrap();
-        assert_eq!(loaded_transaction.accounts[0].lamports, min_balance);
+        assert_eq!(loaded_transaction.accounts[0].lamports(), min_balance);
 
         // Fee leaves zero balance fails
-        accounts[0].1.lamports = min_balance;
+        accounts[0].1.set_lamports(min_balance);
         let loaded_accounts = load_accounts_with_fee_and_rent(
             tx.clone(),
             &accounts,
@@ -1355,7 +1364,7 @@ mod tests {
         assert_eq!(*load_res, Err(TransactionError::InsufficientFundsForFee));
 
         // Fee leaves non-zero, but sub-min_balance balance fails
-        accounts[0].1.lamports = 3 * min_balance / 2;
+        accounts[0].1.set_lamports(3 * min_balance / 2);
         let loaded_accounts = load_accounts_with_fee_and_rent(
             tx,
             &accounts,
@@ -1378,11 +1387,11 @@ mod tests {
         let key1 = Pubkey::new(&[5u8; 32]);
 
         let mut account = AccountSharedData::new(1, 0, &Pubkey::default());
-        account.rent_epoch = 1;
+        account.set_rent_epoch(1);
         accounts.push((key0, account));
 
         let mut account = AccountSharedData::new(2, 1, &Pubkey::default());
-        account.rent_epoch = 1;
+        account.set_rent_epoch(1);
         accounts.push((key1, account));
 
         let instructions = vec![CompiledInstruction::new(2, &(), vec![0, 1])];
@@ -1427,33 +1436,33 @@ mod tests {
         accounts.push((key0, account));
 
         let mut account = AccountSharedData::new(40, 1, &Pubkey::default());
-        account.executable = true;
-        account.owner = native_loader::id();
+        account.set_executable(true);
+        account.set_owner(native_loader::id());
         accounts.push((key1, account));
 
         let mut account = AccountSharedData::new(41, 1, &Pubkey::default());
-        account.executable = true;
-        account.owner = key1;
+        account.set_executable(true);
+        account.set_owner(key1);
         accounts.push((key2, account));
 
         let mut account = AccountSharedData::new(42, 1, &Pubkey::default());
-        account.executable = true;
-        account.owner = key2;
+        account.set_executable(true);
+        account.set_owner(key2);
         accounts.push((key3, account));
 
         let mut account = AccountSharedData::new(43, 1, &Pubkey::default());
-        account.executable = true;
-        account.owner = key3;
+        account.set_executable(true);
+        account.set_owner(key3);
         accounts.push((key4, account));
 
         let mut account = AccountSharedData::new(44, 1, &Pubkey::default());
-        account.executable = true;
-        account.owner = key4;
+        account.set_executable(true);
+        account.set_owner(key4);
         accounts.push((key5, account));
 
         let mut account = AccountSharedData::new(45, 1, &Pubkey::default());
-        account.executable = true;
-        account.owner = key5;
+        account.set_executable(true);
+        account.set_owner(key5);
         accounts.push((key6, account));
 
         let instructions = vec![CompiledInstruction::new(1, &(), vec![0])];
@@ -1488,7 +1497,7 @@ mod tests {
         accounts.push((key0, account));
 
         let mut account = AccountSharedData::new(40, 1, &native_loader::id());
-        account.executable = true;
+        account.set_executable(true);
         accounts.push((key1, account));
 
         let instructions = vec![CompiledInstruction::new(0, &(), vec![0])];
@@ -1523,7 +1532,7 @@ mod tests {
         accounts.push((key0, account));
 
         let mut account = AccountSharedData::new(40, 1, &Pubkey::default());
-        account.executable = true;
+        account.set_executable(true);
         accounts.push((key1, account));
 
         let instructions = vec![CompiledInstruction::new(1, &(), vec![0])];
@@ -1590,19 +1599,19 @@ mod tests {
         let key2 = Pubkey::new(&[6u8; 32]);
 
         let mut account = AccountSharedData::new(1, 0, &Pubkey::default());
-        account.rent_epoch = 1;
+        account.set_rent_epoch(1);
         accounts.push((key0, account));
 
         let mut account = AccountSharedData::new(40, 1, &Pubkey::default());
-        account.executable = true;
-        account.rent_epoch = 1;
-        account.owner = native_loader::id();
+        account.set_executable(true);
+        account.set_rent_epoch(1);
+        account.set_owner(native_loader::id());
         accounts.push((key1, account));
 
         let mut account = AccountSharedData::new(41, 1, &Pubkey::default());
-        account.executable = true;
-        account.rent_epoch = 1;
-        account.owner = key1;
+        account.set_executable(true);
+        account.set_rent_epoch(1);
+        account.set_owner(key1);
         accounts.push((key2, account));
 
         let instructions = vec![
@@ -2010,7 +2019,6 @@ mod tests {
             &(Hash::default(), FeeCalculator::default()),
             true,
             true, // demote_sysvar_write_locks
-            true, // merge_nonce_error_into_system_error
         );
         assert_eq!(collected_accounts.len(), 2);
         assert!(collected_accounts
@@ -2049,12 +2057,12 @@ mod tests {
             false,
         );
         let mut old_pubkey = Pubkey::default();
-        let zero_account = AccountSharedData::new(0, 0, &AccountSharedData::default().owner);
+        let zero_account = AccountSharedData::new(0, 0, AccountSharedData::default().owner());
         info!("storing..");
         for i in 0..2_000 {
             let pubkey = solana_sdk::pubkey::new_rand();
             let account =
-                AccountSharedData::new((i + 1) as u64, 0, &AccountSharedData::default().owner);
+                AccountSharedData::new((i + 1) as u64, 0, AccountSharedData::default().owner());
             accounts.store_slow_uncached(i, &pubkey, &account);
             accounts.store_slow_uncached(i, &old_pubkey, &zero_account);
             old_pubkey = pubkey;
@@ -2064,7 +2072,7 @@ mod tests {
             }
         }
         info!("done..cleaning..");
-        accounts.accounts_db.clean_accounts(None);
+        accounts.accounts_db.clean_accounts(None, false);
     }
 
     fn load_accounts_no_store(accounts: &Accounts, tx: Transaction) -> Vec<TransactionLoadResult> {
@@ -2142,23 +2150,18 @@ mod tests {
         account: &mut AccountSharedData,
         account_pubkey: &Pubkey,
         tx_result: &Result<()>,
-        maybe_nonce_rollback: Option<(
-            &Pubkey,
-            &AccountSharedData,
-            Option<&AccountSharedData>,
-            bool,
-        )>,
+        maybe_nonce_rollback: Option<(&Pubkey, &AccountSharedData, Option<&AccountSharedData>)>,
         last_blockhash_with_fee_calculator: &(Hash, FeeCalculator),
         expect_account: &AccountSharedData,
     ) -> bool {
         // Verify expect_account's relationship
         match maybe_nonce_rollback {
-            Some((nonce_pubkey, _nonce_account, _maybe_fee_account, _))
+            Some((nonce_pubkey, _nonce_account, _maybe_fee_account))
                 if nonce_pubkey == account_pubkey && tx_result.is_ok() =>
             {
                 assert_eq!(expect_account, account) // Account update occurs in system_instruction_processor
             }
-            Some((nonce_pubkey, nonce_account, _maybe_fee_account, _))
+            Some((nonce_pubkey, nonce_account, _maybe_fee_account))
                 if nonce_pubkey == account_pubkey =>
             {
                 assert_ne!(expect_account, nonce_account)
@@ -2172,7 +2175,7 @@ mod tests {
             tx_result,
             maybe_nonce_rollback,
             last_blockhash_with_fee_calculator,
-            false,
+            true,
         );
         expect_account == account
     }
@@ -2202,8 +2205,7 @@ mod tests {
             Some((
                 &pre_account_pubkey,
                 &pre_account,
-                maybe_fee_account.as_ref(),
-                false,
+                maybe_fee_account.as_ref()
             )),
             &(last_blockhash, last_fee_calculator),
             &expect_account,
@@ -2254,8 +2256,7 @@ mod tests {
             Some((
                 &pre_account_pubkey,
                 &pre_account,
-                maybe_fee_account.as_ref(),
-                true,
+                maybe_fee_account.as_ref()
             )),
             &(last_blockhash, last_fee_calculator),
             &expect_account,
@@ -2295,8 +2296,7 @@ mod tests {
             Some((
                 &pre_account_pubkey,
                 &pre_account,
-                maybe_fee_account.as_ref(),
-                true,
+                maybe_fee_account.as_ref()
             )),
             &(last_blockhash, last_fee_calculator),
             &expect_account,
@@ -2396,7 +2396,6 @@ mod tests {
             &(next_blockhash, FeeCalculator::default()),
             true,
             true, // demote_sysvar_write_locks
-            true, // merge_nonce_error_into_system_error
         );
         assert_eq!(collected_accounts.len(), 2);
         assert_eq!(
@@ -2414,7 +2413,10 @@ mod tests {
             .map(|(_pubkey, account)| *account)
             .cloned()
             .unwrap();
-        assert_eq!(collected_nonce_account.lamports, nonce_account_pre.lamports,);
+        assert_eq!(
+            collected_nonce_account.lamports(),
+            nonce_account_pre.lamports(),
+        );
         assert!(nonce_account::verify_nonce_account(
             &collected_nonce_account,
             &next_blockhash
@@ -2513,7 +2515,6 @@ mod tests {
             &(next_blockhash, FeeCalculator::default()),
             true,
             true, // demote_sysvar_write_locks
-            true, // merge_nonce_error_into_system_error
         );
         assert_eq!(collected_accounts.len(), 1);
         let collected_nonce_account = collected_accounts
@@ -2522,7 +2523,10 @@ mod tests {
             .map(|(_pubkey, account)| *account)
             .cloned()
             .unwrap();
-        assert_eq!(collected_nonce_account.lamports, nonce_account_pre.lamports);
+        assert_eq!(
+            collected_nonce_account.lamports(),
+            nonce_account_pre.lamports()
+        );
         assert!(nonce_account::verify_nonce_account(
             &collected_nonce_account,
             &next_blockhash
