@@ -3,8 +3,8 @@ use crate::{
     system_instruction_processor,
 };
 use solana_sdk::{
+    feature_set,
     instruction::InstructionError,
-    keyed_account::KeyedAccount,
     process_instruction::{stable_log, InvokeContext, ProcessInstructionWithContext},
     pubkey::Pubkey,
     system_program,
@@ -13,14 +13,13 @@ use solana_sdk::{
 fn process_instruction_with_program_logging(
     process_instruction: ProcessInstructionWithContext,
     program_id: &Pubkey,
-    keyed_accounts: &[KeyedAccount],
     instruction_data: &[u8],
     invoke_context: &mut dyn InvokeContext,
 ) -> Result<(), InstructionError> {
     let logger = invoke_context.get_logger();
     stable_log::program_invoke(&logger, program_id, invoke_context.invoke_depth());
 
-    let result = process_instruction(program_id, keyed_accounts, instruction_data, invoke_context);
+    let result = process_instruction(program_id, instruction_data, invoke_context);
 
     match &result {
         Ok(()) => stable_log::program_success(&logger, program_id),
@@ -31,14 +30,10 @@ fn process_instruction_with_program_logging(
 
 macro_rules! with_program_logging {
     ($process_instruction:expr) => {
-        |program_id: &Pubkey,
-         keyed_accounts: &[KeyedAccount],
-         instruction_data: &[u8],
-         invoke_context: &mut dyn InvokeContext| {
+        |program_id: &Pubkey, instruction_data: &[u8], invoke_context: &mut dyn InvokeContext| {
             process_instruction_with_program_logging(
                 $process_instruction,
                 program_id,
-                keyed_accounts,
                 instruction_data,
                 invoke_context,
             )
@@ -69,11 +64,6 @@ fn genesis_builtins() -> Vec<Builtin> {
             solana_config_program::id(),
             with_program_logging!(solana_config_program::config_processor::process_instruction),
         ),
-        Builtin::new(
-            "secp256k1_program",
-            solana_sdk::secp256k1_program::id(),
-            solana_secp256k1_program::process_instruction,
-        ),
     ]
 }
 
@@ -92,7 +82,15 @@ pub enum ActivationType {
 /// normal child Bank creation.
 /// https://github.com/solana-labs/solana/blob/84b139cc94b5be7c9e0c18c2ad91743231b85a0d/runtime/src/bank.rs#L1723
 fn feature_builtins() -> Vec<(Builtin, Pubkey, ActivationType)> {
-    vec![]
+    vec![(
+        Builtin::new(
+            "secp256k1_program",
+            solana_sdk::secp256k1_program::id(),
+            solana_secp256k1_program::process_instruction,
+        ),
+        feature_set::secp256k1_program_enabled::id(),
+        ActivationType::NewProgram,
+    )]
 }
 
 pub(crate) fn get() -> Builtins {
